@@ -1,5 +1,5 @@
 /* 8BitAmEthernet
- * version 0.3.1 date 2006-03-12
+ * version 0.3.2 date 2006-04-08
  * Copyright 2005-2006 Stefan Schuermans <1stein@schuermans.info>
  * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  * a project of CCC-AC - http://www.cccac.de/
@@ -135,10 +135,10 @@ void ArpRecv( unsigned char * pData, unsigned short Length ) //(extern)
     ip_cpy( ArpReply.ArpHdr.DestIp, pArpPack->ArpHdr.SrcIp ); //requestor's IP
 
     debug_arp_printf( "recv req src=%02X:%02X:%02X:%02X:%02X:%02X ip=%u.%u.%u.%u",
-                      ArpReply.ArpHdr.SrcMac[0], ArpReply.ArpHdr.SrcMac[1], ArpReply.ArpHdr.SrcMac[2],
-                      ArpReply.ArpHdr.SrcMac[3], ArpReply.ArpHdr.SrcMac[4], ArpReply.ArpHdr.SrcMac[5],
-                      ArpReply.ArpHdr.SrcIp[0], ArpReply.ArpHdr.SrcIp[1], 
-                      ArpReply.ArpHdr.SrcIp[2], ArpReply.ArpHdr.SrcIp[3] );
+                      pArpPack->ArpHdr.SrcMac[0], pArpPack->ArpHdr.SrcMac[1], pArpPack->ArpHdr.SrcMac[2],
+                      pArpPack->ArpHdr.SrcMac[3], pArpPack->ArpHdr.SrcMac[4], pArpPack->ArpHdr.SrcMac[5],
+                      pArpPack->ArpHdr.SrcIp[0], pArpPack->ArpHdr.SrcIp[1], 
+                      pArpPack->ArpHdr.SrcIp[2], pArpPack->ArpHdr.SrcIp[3] );
 
     //sent ARP reply
     mac_cpy( ArpReply.EthHdr.Dest, ArpReply.ArpHdr.DestMac ); //ethernet destination address
