@@ -1,5 +1,5 @@
 /* 8BitAmEthernet
- * version 0.3.1 date 2006-03-12
+ * version 0.3.2 date 2006-04-08
  * Copyright 2005-2006 Stefan Schuermans <1stein@schuermans.info>
  * Copyleft: GNU public license - http://www.gnu.org/copyleft/gpl.html
  * a project of CCC-AC - http://www.cccac.de/
@@ -110,6 +110,7 @@
           RTL_DATA = (val); /* output value */ \
           RTL_WRITE_ACT( ); /* activate write signal */ \
           nop( ); \
+          nop( ); \
           RTL_WRITE_IDLE( ); /* take back write signal */ \
         }
 
@@ -120,6 +121,7 @@
           RTL_ADDR = ((reg) & RTL_ADDR_AND_MASK) | RTL_ADDR_OR_MASK; /* output address */ \
           RTL_READ_ACT( ); /* activate read signal */ \
           nop( ); \
+          nop( ); \
           var = RTL_DATA_IN; /* read data */ \
           RTL_READ_IDLE( ); /* take back read signal */ \
         }
@@ -238,33 +240,22 @@ void RtlTask( void ) //(extern)
 {
   unsigned char isr, bnry, curr;
 
-  //use current timestamp for generating some entropy
-  TimingEntropy( );
-
   //read interrupt register
   RTL_WRITE( RTL_REG_CR, 1<<RTL_CR_STA | 1<<RTL_CR_RD2 );
   RTL_READ( RTL_REG_ISR, isr );
-
   //a packet was received
   if( isr & 1<<RTL_ISR_PRX )
-  {
-    //receive packets
-    for( ; ; )
-    {
-      //get values BNRY, BNRY1 and CURR
-      RTL_READ( RTL_REG_BNRY, bnry );
-      RTL_WRITE( RTL_REG_CR, 1<<RTL_CR_STA | 1<<RTL_CR_RD2 | 1<<RTL_CR_PS0 );
-      RTL_READ( RTL_REG_CURR, curr );
-      RTL_WRITE( RTL_REG_CR, 1<<RTL_CR_STA | 1<<RTL_CR_RD2 );
+    //use current timestamp for generating some entropy
+    TimingEntropy( );
 
-      //receive ring is empty
-      if( bnry == curr )
-        break;
-
-      //fetch and process received packet
-      RtlRecv( );
-    }
-  }
+  //check if receive ring buffer is not empty
+  RTL_READ( RTL_REG_BNRY, bnry );
+  RTL_WRITE( RTL_REG_CR, 1<<RTL_CR_STA | 1<<RTL_CR_RD2 | 1<<RTL_CR_PS0 );
+  RTL_READ( RTL_REG_CURR, curr );
+  RTL_WRITE( RTL_REG_CR, 1<<RTL_CR_STA | 1<<RTL_CR_RD2 );
+  if( bnry != curr )
+    //fetch and process a single received packet
+    RtlRecv( );
 
   //reset interrupt bits of RTL8019
   RTL_WRITE( RTL_REG_ISR, 1<<RTL_ISR_PRX | 1<<RTL_ISR_PTX |
@@ -272,7 +263,7 @@ void RtlTask( void ) //(extern)
                           1<<RTL_ISR_OVW | 1<<RTL_ISR_CNT |
                           1<<RTL_ISR_RDC | 1<<RTL_ISR_RST );
 
-  //start RTL8019
+  //put RTL8019 in default state (default page selected, ...)
   RTL_WRITE( RTL_REG_CR, 1<<RTL_CR_STA | 1<<RTL_CR_RD2 );
 
   //clear reinitialization timeout
@@ -332,7 +323,7 @@ void RtlWriteFrame( char * pData, unsigned short Length ) //(extern)
     RTL_WRITE( RTL_REG_TBCR0, (unsigned char)Length );
     RTL_WRITE( RTL_REG_TBCR1, (unsigned char)(Length >> 8) );
   }
-  RTL_WRITE( RTL_REG_CR, 1<<RTL_CR_TXP| 1<<RTL_CR_RD2 );
+  RTL_WRITE( RTL_REG_CR, 1<<RTL_CR_TXP | 1<<RTL_CR_RD2 );
 }
 
 //read an ethernet frame from the RTL8019
@@ -340,7 +331,7 @@ void RtlWriteFrame( char * pData, unsigned short Length ) //(extern)
 void RtlReadFrame( char * pData, unsigned short * pLength ) //(extern)
 {
   unsigned char tmp1, tmp2;
-  unsigned short len, cnt, i;
+  unsigned short read_len, len, cnt, i;
 
   //get size of received packet
   RTL_WRITE( RTL_REG_CR, 1<<RTL_CR_STA | 1<<RTL_CR_RD0 | 1<<RTL_CR_RD1 );
@@ -348,7 +339,10 @@ void RtlReadFrame( char * pData, unsigned short * pLength ) //(extern)
   RTL_READ( RTL_REG_RDMAPORT, tmp1 );
   RTL_READ( RTL_REG_RDMAPORT, tmp1 );
   RTL_READ( RTL_REG_RDMAPORT, tmp2 );
-  len = ((unsigned short)tmp2 << 8 | (unsigned short)tmp1);
+  read_len = ((unsigned short)tmp2 << 8 | (unsigned short)tmp1);
+
+  //subtract length of CRC (4 bytes)
+  len = read_len < 4 ? 0 : read_len - 4;
 
   debug_rtl8019_printf( "recv len=%u", len );
 
@@ -358,11 +352,8 @@ void RtlReadFrame( char * pData, unsigned short * pLength ) //(extern)
     RTL_READ( RTL_REG_RDMAPORT, pData[i] );
   *pLength = cnt;
 
-  //get rest of data
-  for( ; i < len; i++ )
+  //get rest of data (rest of real data that did not fit into buffer, CRC)
+  for( ; i < read_len; i++ )
     RTL_READ( RTL_REG_RDMAPORT, tmp1 );
-
-  //subtract length of CRC (4 bytes)
-  len = len < 4 ? 0 : len - 4;
 }
 
